==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ronportal/pokladnikniha/pokladnikniha.php <==
<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 8.6.2016
 * Time: 12:41
 */

namespace RonPortal\PokladniKniha;

use Exception;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use RonPortal\Prava;
use RonPortal\Rozdeleni;
use function Ron\ronText;

class PokladniKniha
{
    public $datumPohybu;
    public $osobniCislo;
    public $castka;
    public $mena;
    public $typPolozky;
    public $popis;
    public $poznamka;
    /** @var int[] datum a cas schvaleni dle urovne */
    public $schvaleni;
    public $schvalil;
    /** @var int[] datum a cas vraceni dle urovne */
    public $datumVraceni;
    public $poznamkaKVraceni;
    public $historieSchvalovani;
    public $obdobiVyuctovani;
    public $idPP;
    public $castkaBezDPH;
    public $castkaDPH;
    public $cisloBrowseru;

    public $zauctovano;
    public $zauctovanoStr;

    /** @var Rozdeleni */
    public $rozdeleni;
    public $typRozdeleniPopis;
    /** @var Rozdeleni */
    public $rozdeleni2;
    public $typRozdeleni2Popis;
    /** @var Rozdeleni */
    public $rozdeleni3;
    public $typRozdeleni3Popis;
    /** @var Rozdeleni */
    public $rozdeleni4;
    public $typRozdeleni4Popis;

    public $existujiRadky = false;

    public $typPolozkyKod;
    private $id;
    private $rc;
    private $jmeno;
    private $prijmeni;
    private $titul;

    public function __construct()
    {
        $this->schvaleni = [];
        $this->schvalil = [];
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->rc = $row[2];
        $this->jmeno = $row[22];
        $this->prijmeni = $row[23];
        $this->titul = $row[24];

        $this->datumPohybu = $database->stringToTimestamp($row[1]);
        $this->osobniCislo = $row[3];
        $this->castka = $row[5];
        $this->mena = $row[6];
        $this->typPolozky = $row[20];
        $this->popis = $row[8];
        $this->poznamka = $row[9];
        $this->schvaleni[1] = $database->stringToTimestamp($row[11]);
        $this->schvalil[1] = $row[12];
        $this->schvaleni[2] = $database->stringToTimestamp($row[13]);
        $this->schvalil[2] = $row[14];
        $this->schvaleni[3] = $database->stringToTimestamp($row[15]);
        $this->schvalil[3] = $row[16];
        $this->schvaleni[4] = $database->stringToTimestamp($row[38]);
        $this->schvalil[4] = $row[39];

        if (PokladniKnihaController::existujeDatumVraceni($database)) {
            for ($i = 1; $i <= 4; $i++) {
                $this->datumVraceni[$i] = $database->stringToTimestamp($row[50 + $i]);
            }
        }

        $this->poznamkaKVraceni = $row[17];
        $this->historieSchvalovani = $row[18];
        $this->obdobiVyuctovani = $row[19];
        $this->idPP = $row[21];

        $this ->zauctovano = $row[40];
        $this ->zauctovanoStr = $this ->zauctovano
            ? ronText('PoklK_zauctovana') . ' (' . DBUtils::getFormatedValue($database, 'finedatetime', $row[42]) . ', ' . $row[41] . ')'
            : '';

        $this->castkaBezDPH = $row[25];
        $this->castkaDPH = $row[26];
        $this->typPolozkyKod = $row[27];
        $this->cisloBrowseru = $row[28] ?: 1;

        if ($row[30]) {
            $this->rozdeleni = new Rozdeleni();
            $this->rozdeleni->typ = $row[29];
            $this->rozdeleni->kod = $row[30];
            $this->rozdeleni->popis = $row[33];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[29]}", "");
            $this->typRozdeleniPopis = $popisTypu ?: $row[35];
        }

        if ($row[32]) {
            $this->rozdeleni2 = new Rozdeleni();
            $this->rozdeleni2->typ = $row[31];
            $this->rozdeleni2->kod = $row[32];
            $this->rozdeleni2->popis = $row[34];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[31]}", "");
            $this->typRozdeleni2Popis = $popisTypu ?: $row[36];
        }

        if ($row[43]) {
            $this->rozdeleni3 = new Rozdeleni();
            $this->rozdeleni3->typ = $row[43];
            $this->rozdeleni3->kod = $row[44];
            $this->rozdeleni3->popis = $row[46];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[43]}", "");
            $this->typRozdeleni3Popis = $popisTypu ?: $row[45];
        }

        if ($row[47]) {
            $this->rozdeleni4 = new Rozdeleni();
            $this->rozdeleni4->typ = $row[47];
            $this->rozdeleni4->kod = $row[48];
            $this->rozdeleni4->popis = $row[50];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[47]}", "");
            $this->typRozdeleni4Popis = $popisTypu ?: $row[49];
        }

        $this->existujiRadky = ($row[37] !== 0);
    }

    /**
     * @param Database $database
     * @param int $id id zaznamu
     * @return PokladniKniha|false
     * @throws DBException
     */
    public static function nactiZaznam($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, ' . //0 - 4
            ' PokladniKniha.Castka, PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, PokladniKniha.IDDokumentu, ' . //5 - 10
            ' PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, ' . //11 - 16
            ' PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, PokladniKniha.ObdobiVyuctovani, TypPolozkyPokladniKnihy.Popis, ' . //17 - 20
            ' PracovniPomer.ID, Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, ' . //21 - 24
            ' PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH, TypPolozkyPokladniKnihy.Kod, TypPolozkyPokladniKnihy.RonPortalBrowser, ' . //25 - 28
            ' PokladniKniha.KodTypuRozdeleni, PokladniKniha.Rozdeleni, PokladniKniha.KodTypuRozdeleni2, PokladniKniha.Rozdeleni2, Rozdeleni.Popis, R2.Popis,  ' .  //29 - 34
            ' TypRozdeleni.Popis, TR2.Popis, (SELECT COUNT(CisloRadku) FROM RadekPokladniKnihy WHERE RadekPokladniKnihy.IDPokladniKnihy = PokladniKniha.ID), ' . //35 - 37
            ' PokladniKniha.Schvaleni4, PokladniKniha.Schvalil4, PokladniKniha.Zauctovano, PokladniKniha.Zauctovano_Uzivatel, PokladniKniha.Zauctovano_DatumCas, ' . // 38 - 42
            ' PokladniKniha.KodTypuRozdeleni3, PokladniKniha.Rozdeleni3, TR3.Popis, R3.Popis ' ;  //43 - 46

        if (PokladniKnihaController::zobrazitRozdeleni4($database)) {
            $sql .=
                ' ,PokladniKniha.KodTypuRozdeleni4, PokladniKniha.Rozdeleni4, TR4.Popis, R4.Popis ' ; //47 - 50
        }

        if (PokladniKnihaController::existujeDatumVraceni($database)) {
            $sql .=
                ' ,PokladniKniha.DatumVraceni1, PokladniKniha.DatumVraceni2, PokladniKniha.DatumVraceni3, PokladniKniha.DatumVraceni4 ' ; // 51 - 54
        }

        $sql .=
            'FROM PokladniKniha JOIN TypPolozkyPokladniKnihy ON (TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky)' .
            ' JOIN PracovniPomer ON (PracovniPomer.RC = PokladniKniha.RC AND PracovniPomer.OsobniCislo = PokladniKniha.OsobniCislo AND PracovniPomer.DatumNastupu = PokladniKniha.DatumNastupu) ' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC) ' .
            ' LEFT OUTER JOIN Rozdeleni ON Rozdeleni.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni AND Rozdeleni.Kod = PokladniKniha.Rozdeleni ' .
            ' LEFT OUTER JOIN Rozdeleni R2 ON R2.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni2 AND R2.Kod = PokladniKniha.Rozdeleni2 ' .
            ' LEFT OUTER JOIN Rozdeleni R3 ON R3.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni3 AND R3.Kod = PokladniKniha.Rozdeleni3 ' .
            ' LEFT OUTER JOIN Rozdeleni R4 ON R4.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni4 AND R4.Kod = PokladniKniha.Rozdeleni4 ' .
            ' LEFT OUTER JOIN TypRozdeleni ON TypRozdeleni.Kod = Rozdeleni.KodTypuRozdeleni ' .
            ' LEFT OUTER JOIN TypRozdeleni TR2 ON TR2.Kod = R2.KodTypuRozdeleni ' .
            ' LEFT OUTER JOIN TypRozdeleni TR3 ON TR3.Kod = R3.KodTypuRozdeleni ' .
            ' LEFT OUTER JOIN TypRozdeleni TR4 ON TR4.Kod = R4.KodTypuRozdeleni ' .
            'WHERE PokladniKniha.ID = ' . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            $zaznam = new PokladniKniha();
            $zaznam->nactiDataZRadku($database, $row);
            return $zaznam;
        }

        return false;
    }

    /**
     * @return string Prijmeni Jmeno, Titul
     */
    public function vratJmenoPrijmeni()
    {
        return $this->prijmeni . ' ' . $this->jmeno . ($this->titul ? ', ' . $this->titul : '');
    }

    /**
     * @return int id zaznamu pokladni knihy
     */
    public function vratID()
    {
        return $this->id;
    }

    /** Vraci true, pokud je zaznam schvalen nejvyssi urovni, na kterou ma uzivatel pravo, nebo vyssi urovni
     * @return bool
     * @throws Exception
     */
    public function jeSchvalenaProUzivatele()
    {
        if ($this->schvaleni[4]) {
            // nejvyssi uroven zamyka vzdy
            return true;
        }

        if (Prava::vratPrava('pkniha_schval4', 'pkniha_schval4_' . $this->typPolozkyKod)->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
            // ma pravo na uroven 4
            return false;
        }

        if ($this ->schvaleni[3]) {
            return true;
        }

        if (Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $this->typPolozkyKod)->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
            // ma pravo na uroven 3
            return false;
        }

        if ($this ->schvaleni[2]) {
            return true;
        }

        if (Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $this->typPolozkyKod)->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
            // ma pravo na uroven 2
            return false;
        }

        return $this ->schvaleni[1];
    }

    /**
     * @param bool $raise vyhazovat vyjimku
     * @return bool
     * @throws Exception
     */
    public function lzeEditovat($raise = true)
    {
        try {
            // kontrola zauctovani - nelze editovat zauctovane
            if ($this ->zauctovano) {
                throw new Exception(ronText('PoklK_jiz_zauctovano') );
            }

            // kontrola schvaleni (nelze editovat schvalene, ale pokud mam pravo na vyssi uroven, tak muzu)
            $schvalena = $this ->jeSchvalenaProUzivatele();

            if ($schvalena) {
                throw new Exception(ronText('PoklK_zaznam_schvalen'));
            }

            // kontrola uzavreneho obdobi??
            /*if (Obdobi::jeObdobiUzavreno($database, 'ADS', $this ->datumCasOd, $this ->datumCasDo)) {
                if ($raise) {
                    throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
                }

                return false;
            }*/

            // kontrola prav
            $prava = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $this->typPolozkyKod);
            if (!$prava->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
                throw new Exception(ronText('Neni_pravo_editovat_poklk'));
            }
        }
        catch (Exception $exception) {
            if ($raise) {
                throw $exception;
            }

            return false;
        }

        return true;
    }

    /**
     * Vraci true, pokud je zaznam schvalen zadanym levelem nebo vyssim
     * @param $minimalniLevel
     * @return bool
     */
    public function jeZaznamSchvalen($minimalniLevel = 1)
    {
        if ($this ->schvaleni[4]) {
            return true;
        }

        if ($this ->schvaleni[3]) return $minimalniLevel <= 3;
        if ($this ->schvaleni[2]) return $minimalniLevel <= 2;
        if ($this ->schvaleni[1]) return $minimalniLevel <= 1;

        return false;
    }

    /**
     * @param Database $database
     * @param int $id
     * @return Cursor 0 = id pokladni knihy, 1 = id dokumentu
     * @throws DBException
     */
    public static function nactiPripojeneDokumenty($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT IDPokladniKnihy, ID 
             FROM DokumentPokladniKnihy 
             WHERE IDPokladniKnihy = ' . $params->addParameter((int)$id);

        return $database->getSqlResult($sql, $params);
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/ronportal/pokladnikniha/pokladnikniha.php <==
<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 8.6.2016
 * Time: 12:41
 */

namespace RonPortal\PokladniKniha;

use Exception;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use RonPortal\Prava;
use RonPortal\Rozdeleni;
use function Ron\ronText;

class PokladniKniha
{
    public $datumPohybu;
    public $osobniCislo;
    public $castka;
    public $mena;
    public $typPolozky;
    public $popis;
    public $poznamka;
    /** @var int[] datum a cas schvaleni dle urovne */
    public $schvaleni;
    public $schvalil;
    /** @var int[] datum a cas vraceni dle urovne */
    public $datumVraceni;
    public $poznamkaKVraceni;
    public $historieSchvalovani;
    public $obdobiVyuctovani;
    public $idPP;
    public $castkaBezDPH;
    public $castkaDPH;
    public $cisloBrowseru;

    public $zauctovano;
    public $zauctovanoStr;

    /** @var Rozdeleni */
    public $rozdeleni;
    public $typRozdeleniPopis;
    /** @var Rozdeleni */
    public $rozdeleni2;
    public $typRozdeleni2Popis;
    /** @var Rozdeleni */
    public $rozdeleni3;
    public $typRozdeleni3Popis;
    /** @var Rozdeleni */
    public $rozdeleni4;
    public $typRozdeleni4Popis;

    public $existujiRadky = false;

    public $zpusobPlatby;
    public $zpusobPlatbyPopis;

    public $typPolozkyKod;
    private $id;
    private $rc;
    private $jmeno;
    private $prijmeni;
    private $titul;

    public function __construct()
    {
        $this->schvaleni = [];
        $this->schvalil = [];
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->id = $row[0];
        $this->rc = $row[2];
        $this->jmeno = $row[22];
        $this->prijmeni = $row[23];
        $this->titul = $row[24];

        $this->datumPohybu = $database->stringToTimestamp($row[1]);
        $this->osobniCislo = $row[3];
        $this->castka = $row[5];
        $this->mena = $row[6];
        $this->typPolozky = $row[20];
        $this->popis = $row[8];
        $this->poznamka = $row[9];
        $this->schvaleni[1] = $database->stringToTimestamp($row[11]);
        $this->schvalil[1] = $row[12];
        $this->schvaleni[2] = $database->stringToTimestamp($row[13]);
        $this->schvalil[2] = $row[14];
        $this->schvaleni[3] = $database->stringToTimestamp($row[15]);
        $this->schvalil[3] = $row[16];
        $this->schvaleni[4] = $database->stringToTimestamp($row[38]);
        $this->schvalil[4] = $row[39];

        if (PokladniKnihaController::existujeDatumVraceni($database)) {
            for ($i = 1; $i <= 4; $i++) {
                $this->datumVraceni[$i] = $database->stringToTimestamp($row[52 + $i]);
            }
        }

        $this->poznamkaKVraceni = $row[17];
        $this->historieSchvalovani = $row[18];
        $this->obdobiVyuctovani = $row[19];
        $this->idPP = $row[21];

        $this ->zauctovano = $row[40];
        $this ->zauctovanoStr = $this ->zauctovano
            ? ronText('PoklK_zauctovana') . ' (' . DBUtils::getFormatedValue($database, 'finedatetime', $row[42]) . ', ' . $row[41] . ')'
            : '';

        $this->castkaBezDPH = $row[25];
        $this->castkaDPH = $row[26];
        $this->typPolozkyKod = $row[27];
        $this->cisloBrowseru = $row[28] ?: 1;

        if ($row[30]) {
            $this->rozdeleni = new Rozdeleni();
            $this->rozdeleni->typ = $row[29];
            $this->rozdeleni->kod = $row[30];
            $this->rozdeleni->popis = $row[33];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[29]}", "");
            $this->typRozdeleniPopis = $popisTypu ?: $row[35];
        }

        if ($row[32]) {
            $this->rozdeleni2 = new Rozdeleni();
            $this->rozdeleni2->typ = $row[31];
            $this->rozdeleni2->kod = $row[32];
            $this->rozdeleni2->popis = $row[34];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[31]}", "");
            $this->typRozdeleni2Popis = $popisTypu ?: $row[36];
        }

        if ($row[43]) {
            $this->rozdeleni3 = new Rozdeleni();
            $this->rozdeleni3->typ = $row[43];
            $this->rozdeleni3->kod = $row[44];
            $this->rozdeleni3->popis = $row[46];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[43]}", "");
            $this->typRozdeleni3Popis = $popisTypu ?: $row[45];
        }

        if ($row[49]) {
            $this->rozdeleni4 = new Rozdeleni();
            $this->rozdeleni4->typ = $row[49];
            $this->rozdeleni4->kod = $row[50];
            $this->rozdeleni4->popis = $row[52];

            $popisTypu = ronText("Popis_typu_rozdeleni_{$row[49]}", "");
            $this->typRozdeleni4Popis = $popisTypu ?: $row[51];
        }

        $this->zpusobPlatby = $row[47];
        $this->zpusobPlatbyPopis = $row[48];

        $this->existujiRadky = ($row[37] != 0);
    }

    /**
     * @param Database $database
     * @param int $id id zaznamu
     * @return PokladniKniha|false
     * @throws DBException
     */
    public static function nactiZaznam($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, ' . //0 - 4
            ' PokladniKniha.Castka, PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, PokladniKniha.IDDokumentu, ' . //5 - 10
            ' PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, ' . //11 - 16
            ' PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, PokladniKniha.ObdobiVyuctovani, TypPolozkyPokladniKnihy.Popis, ' . //17 - 20
            ' PracovniPomer.ID, Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, ' . //21 - 24
            ' PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH, TypPolozkyPokladniKnihy.Kod, TypPolozkyPokladniKnihy.RonPortalBrowser, ' . //25 - 28
            ' PokladniKniha.KodTypuRozdeleni, PokladniKniha.Rozdeleni, PokladniKniha.KodTypuRozdeleni2, PokladniKniha.Rozdeleni2, Rozdeleni.Popis, R2.Popis,  ' .  //29 - 34
            ' TypRozdeleni.Popis, TR2.Popis, (SELECT COUNT(CisloRadku) FROM RadekPokladniKnihy WHERE RadekPokladniKnihy.IDPokladniKnihy = PokladniKniha.ID), ' . //35 - 37
            ' PokladniKniha.Schvaleni4, PokladniKniha.Schvalil4, PokladniKniha.Zauctovano, PokladniKniha.Zauctovano_Uzivatel, PokladniKniha.Zauctovano_DatumCas, ' . // 38 - 42
            ' PokladniKniha.KodTypuRozdeleni3, PokladniKniha.Rozdeleni3, TR3.Popis, R3.Popis, ZpusobPlatbyPK.Kod, ZpusobPlatbyPK.Popis ' ;  //43 - 48

        if (PokladniKnihaController::zobrazitRozdeleni4($database)) {
            $sql .=
                ' ,PokladniKniha.KodTypuRozdeleni4, PokladniKniha.Rozdeleni4, TR4.Popis, R4.Popis ' ; //49 - 52
        }

        if (PokladniKnihaController::existujeDatumVraceni($database)) {
            $sql .=
                ' ,PokladniKniha.DatumVraceni1, PokladniKniha.DatumVraceni2, PokladniKniha.DatumVraceni3, PokladniKniha.DatumVraceni4 ' ; // 53 - 56
        }

        $sql .=
            'FROM PokladniKniha JOIN TypPolozkyPokladniKnihy ON (TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky)' .
            ' JOIN PracovniPomer ON (PracovniPomer.RC = PokladniKniha.RC AND PracovniPomer.OsobniCislo = PokladniKniha.OsobniCislo AND PracovniPomer.DatumNastupu = PokladniKniha.DatumNastupu) ' .
            ' JOIN Osoba ON (Osoba.RC = PracovniPomer.RC) ' .
            ' JOIN ZpusobPlatbyPK ON (ZpusobPlatbyPK.Kod = PokladniKniha.ZpusobPlatby) ' .
            ' LEFT OUTER JOIN Rozdeleni ON Rozdeleni.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni AND Rozdeleni.Kod = PokladniKniha.Rozdeleni ' .
            ' LEFT OUTER JOIN Rozdeleni R2 ON R2.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni2 AND R2.Kod = PokladniKniha.Rozdeleni2 ' .
            ' LEFT OUTER JOIN Rozdeleni R3 ON R3.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni3 AND R3.Kod = PokladniKniha.Rozdeleni3 ' .
            ' LEFT OUTER JOIN Rozdeleni R4 ON R4.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni4 AND R4.Kod = PokladniKniha.Rozdeleni4 ' .
            ' LEFT OUTER JOIN TypRozdeleni ON TypRozdeleni.Kod = Rozdeleni.KodTypuRozdeleni ' .
            ' LEFT OUTER JOIN TypRozdeleni TR2 ON TR2.Kod = R2.KodTypuRozdeleni ' .
            ' LEFT OUTER JOIN TypRozdeleni TR3 ON TR3.Kod = R3.KodTypuRozdeleni ' .
            ' LEFT OUTER JOIN TypRozdeleni TR4 ON TR4.Kod = R4.KodTypuRozdeleni ' .
            'WHERE PokladniKniha.ID = ' . $params->addParameter($id);

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            $zaznam = new PokladniKniha();
            $zaznam->nactiDataZRadku($database, $row);
            return $zaznam;
        }

        return false;
    }

    /**
     * @return string Prijmeni Jmeno, Titul
     */
    public function vratJmenoPrijmeni()
    {
        return $this->prijmeni . ' ' . $this->jmeno . ($this->titul ? ', ' . $this->titul : '');
    }

    /**
     * @return int id zaznamu pokladni knihy
     */
    public function vratID()
    {
        return $this->id;
    }

    /** Vraci true, pokud je zaznam schvalen nejvyssi urovni, na kterou ma uzivatel pravo, nebo vyssi urovni
     * @return bool
     * @throws Exception
     */
    public function jeSchvalenaProUzivatele()
    {
        if ($this->schvaleni[4]) {
            // nejvyssi uroven zamyka vzdy
            return true;
        }

        if (Prava::vratPrava('pkniha_schval4', 'pkniha_schval4_' . $this->typPolozkyKod)->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
            // ma pravo na uroven 4
            return false;
        }

        if ($this ->schvaleni[3]) {
            return true;
        }

        if (Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $this->typPolozkyKod)->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
            // ma pravo na uroven 3
            return false;
        }

        if ($this ->schvaleni[2]) {
            return true;
        }

        if (Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $this->typPolozkyKod)->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
            // ma pravo na uroven 2
            return false;
        }

        return $this ->schvaleni[1];
    }

    /**
     * @param bool $raise vyhazovat vyjimku
     * @return bool
     * @throws Exception
     */
    public function lzeEditovat($raise = true)
    {
        try {
            // kontrola zauctovani - nelze editovat zauctovane
            if ($this ->zauctovano) {
                throw new Exception(ronText('PoklK_jiz_zauctovano') );
            }

            // kontrola schvaleni (nelze editovat schvalene, ale pokud mam pravo na vyssi uroven, tak muzu)
            $schvalena = $this ->jeSchvalenaProUzivatele();

            if ($schvalena) {
                throw new Exception(ronText('PoklK_zaznam_schvalen'));
            }

            // kontrola uzavreneho obdobi??
            /*if (Obdobi::jeObdobiUzavreno($database, 'ADS', $this ->datumCasOd, $this ->datumCasDo)) {
                if ($raise) {
                    throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
                }

                return false;
            }*/

            // kontrola prav
            $prava = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $this->typPolozkyKod);
            if (!$prava->lzeZobrazitPP($this->idPP, $this->datumPohybu, $this->datumPohybu)) {
                throw new Exception(ronText('Neni_pravo_editovat_poklk'));
            }
        }
        catch (Exception $exception) {
            if ($raise) {
                throw $exception;
            }

            return false;
        }

        return true;
    }

    /**
     * Vraci true, pokud je zaznam schvalen zadanym levelem nebo vyssim
     * @param $minimalniLevel
     * @return bool
     */
    public function jeZaznamSchvalen($minimalniLevel = 1)
    {
        if ($this ->schvaleni[4]) {
            return true;
        }

        if ($this ->schvaleni[3]) return $minimalniLevel <= 3;
        if ($this ->schvaleni[2]) return $minimalniLevel <= 2;
        if ($this ->schvaleni[1]) return $minimalniLevel <= 1;

        return false;
    }

    /**
     * @param Database $database
     * @param int $id
     * @return Cursor 0 = id pokladni knihy, 1 = id dokumentu
     * @throws DBException
     */
    public static function nactiPripojeneDokumenty($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT IDPokladniKnihy, ID 
             FROM DokumentPokladniKnihy 
             WHERE IDPokladniKnihy = ' . $params->addParameter((int)$id);

        return $database->getSqlResult($sql, $params);
    }
}
